%Author: Emiliano Cristiani, 2013
%Naif direct method for finite horizon problem, 1D

%y'=f(y,a), a in A
%y(0)=x
%Payoff J=int_0^T r(y,a)ds + g(y(T))

clear
clc
close all

global x Nt Dt

%% parameters
x=1;         %initial point
T=2;         %final time
Nt=100;      %time steps
beta=0.2;    %descent step for gradient method
delta=0.05;  %parameter for evaluation of grad J
tol=1e-3;    %tolerance for gradient method
it_max=2000; %maximum number of iteration for gradient method

Dt=T/Nt;  disp(['Dt=',num2str(Dt)]);
for (n=1:Nt) t(n)=(n-1)*Dt;       end

%% ::::::::: PENALIZED GRADIENT METHOD (Fixed Step) :::::::::::

error=tol+1; it=0;
alphaold(1:Nt)=0.5; %initial guess for alpha

graph_alpha=plot(t,alphaold,'ro-','markersize',4);
title('alpha');
axis([0 T -2 2])

while ( (it<it_max) && (error>tol) )
  disp(['it=',num2str(it),', J(alpha)=',num2str(J(alphaold))]);
  it=it+1;

  %grad J
  for n=1:Nt
    perturbation=zeros(1,Nt);
    perturbation(n)=delta;
    alphaplus =alphaold+perturbation;
    alphaminus=alphaold-perturbation;
    gradJ(n)=( J(alphaplus)-J(alphaminus) ) / (2*delta);
  end

  alphanew=alphaold+beta.*gradJ;
  error=max(abs(alphanew-alphaold));
  alphaold=alphanew;

  set(graph_alpha,'YData',alphaold);
  drawnow;
  title(['iteration = ',num2str(it)])
  xlabel('t')
  ylabel('alpha')
  pause(0.01);
end



