%Author: Emiliano Cristiani, 2013
%HJB equation for finite horizon problem, 1D
%semi-Lagrangian scheme, 1st order interpolation

%Vt(x,t) + max_{a in A} {f(x,a) \nabla V(x,t) + r(x,a)}=0
%V(x,T)=g(x)

clear
clc
close all

%% parameters
xmax=4;     %space domain [-xmax,xmax]
T=2;        %time domain [0,T]
Nx=200;     %space steps
Nt=200;     %time steps
Nc=5;       %number of discrete controls
a(1)=-1; a(2)=-0.5; a(3)=0; a(4)=0.5; a(5)=1; %discrete controls in A=[-1,1]

%% initializations
Dt=T/Nt;        disp(['Dt=',num2str(Dt)]);
Dx=(2*xmax)/Nx; disp(['Dx=',num2str(Dx)]);

for (i=1:Nx) x(i)=-xmax+(i-1)*Dx; end
for (n=1:Nt) t(n)=(n-1)*Dt;       end

%matrix initializations and boundary conditions
V(1:Nx,1:Nt)=0; %insert here boundary conditions for V
Vx(1:Nx,1:Nt)=0;
G(1:Nx)=0;
F(1:Nx,1:Nc)=0;
R(1:Nx,1:Nc)=0;
ASTAR(1:Nx,1:Nt)=0;

%dynamics F, running payoff R, final payoff G
for i=1:Nx
  G(i)=g(x(i));
  for k=1:Nc
    F(i,k)=f(x(i),a(k));
    R(i,k)=r(x(i),a(k));
  end
end

%final condition for V
for i=1:Nx
  V(i,Nt)=G(i);
end

%check CFL
velmax=max(max(abs(F)));
CFL=(Dt/Dx)*velmax; disp(['CFL= ',num2str(CFL)]);
if (CFL>1) disp('!!!!! NO CFL'); disp(['CFL= ',num2str(CFL)]); pause; end


%% ::::::::::: COMPUTATION V(x,t) and Vx(x,t) ::::::::::::
for n=Nt:-1:2
  for i=2:Nx-1
    
    for k=1:Nc
      %computation Vinterp=V(xi+hf,tn)
      if F(i,k)==0
        Vinterp=V(i,n);
      elseif F(i,k)>0
        peso=(Dt*F(i,k))/Dx;
        Vinterp=(1-peso)*V(i,n) + peso*V(i+1,n);
      elseif F(i,k)<0
        peso=-(Dt*F(i,k))/Dx;
        Vinterp=(1-peso)*V(i,n) + peso*V(i-1,n);
      else
        disp('error in SLscheme');
      end
      H(k)=Vinterp+Dt*R(i,k);
    end
    
    V(i,n-1)=max(H);
    
  end
end

for n=1:Nt
  for i=2:Nx-1
    Vx(i,n)=(V(i+1,n)-V(i-1,n)) / (2*Dx);
  end
end
%::::::::::: END MAIN CYCLE :::::::::::

%% ::::::::::: COMPUTATION a*(x,t) ::::::::::::
for n=1:Nt
  for i=2:Nx-1
    
    for k=1:Nc
      %computation Vinterp=V(xi+hf,tn)
      if F(i,k)==0
        Vinterp=V(i,n);
      elseif F(i,k)>0
        peso=(Dt*F(i,k))/Dx;
        Vinterp=(1-peso)*V(i,n) + peso*V(i+1,n);
      elseif F(i,k)<0
        peso=-(Dt*F(i,k))/Dx;
        Vinterp=(1-peso)*V(i,n) + peso*V(i-1,n);
      else
        disp('error in SLscheme');
      end
      H(k)=Vinterp+Dt*R(i,k);
    end
    
    [massimo,kstar]=max(H);
    ASTAR(i,n)=a(kstar);
    
  end
end


%% :::::::::::::: GRAPHS ::::::::::::::

%V at time t=0
plot(x,V(:,1),'o-');
xlabel('x'); ylabel('V(x,0)');

%Vx at time t=0
figure
plot(x,Vx(:,1),'ro-');
xlabel('x'); ylabel('Vx(x,0)');

%a* with imagesc
figure
imagesc(x,t,ASTAR');
xlabel('x'); ylabel('t'); title('optimal feedback control a*(x,t)');
break

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%a*(x,t) with mesh
mesh(t,x,ASTAR);
xlabel('t'); ylabel('x'); zlabel('a*');

%V(x,t) with mesh
mesh(t,x,V);
xlabel('t'); ylabel('x'); zlabel('v');

%V(x,t) (animation)
graph_V=plot(x,V(:,1),'o-','markersize',4);
title('V');
axis([-xmax xmax min(min(V)) max(max(V))])

for n=1:1:Nt
  set(graph_V,'YData',V(:,n));
  drawnow;
  xlabel(n*Dt)
  pause(0.01);
end

