%Author: Emiliano Cristiani, 2013
%Naif shooting method for PMP for finite horizon problem, 1D

%y'=f(y,a), a in A
%y(0)=x
%Payoff J=int_0^T r(y,a)ds + g(y(T))

clear
clc
close all

%% parameters
T=2;        %final time
y(1)=0;     %initial point x=y(0)
Nt=101;     %time steps
Nc=5;       %number of discrete controls
a(1)=-1; a(2)=-0.5; a(3)=0; a(4)=0.5; a(5)=1; %discrete controls in A=[-1,1]
singleshoot=false; %plot shooting function S or make a single shoot

Dt=T/(Nt-1);  disp(['Dt=',num2str(Dt)]);
for (n=1:Nt) t(n)=(n-1)*Dt; end

if singleshoot
  p0(1)=2;
  Np0=1;
else
  p0=-3:0.05:3;
  Np0=length(p0);
end


for j=1:Np0

  p(1)=p0(j);

  %::::::::::: COMPUTATION y,p,alpha ::::::::::::
  for n=1:Nt-1

    for k=1:Nc
      H(k)=f(y(n),a(k))*p(n)+r(y(n),a(k));
    end
    [massimo,kstar]=max(H);
    alpha(n)=a(kstar);

    y(n+1)=y(n)+Dt*f(y(n),alpha(n));
    p(n+1)=p(n)+Dt*( -p(n)*fx(y(n),alpha(n)) - rx(y(n),alpha(n)) );
  end
  alpha(Nt)=alpha(Nt-1);
  %:::::::::::::::::::::::::::::::::::::::::::

  S(j)=p(Nt);
end

%% :::::::::::::: GRAPHS ::::::::::::::

if Np0>1
  plot(p0,S,'.-',p0,zeros(1,Np0),'r');
  xlabel('p0');
  ylabel('S');
end

if Np0==1
  figure
  subplot(1,3,1)
  plot(y,t,'o-');
  ylabel('s');
  xlabel('y*');

  subplot(1,3,2)
  plot(p,t,'x-');
  xlabel('p*');

  subplot(1,3,3)
  plot(alpha,t,'.-');
  xlabel('alpha*');
end
